/*
 / _____)             _              | |
( (____  _____ ____ _| |_ _____  ____| |__
 \____ \| ___ |    (_   _) ___ |/ ___)  _ \
 _____) ) ____| | | || |_| ____( (___| | | |
(______/|_____)_|_|_| \__)_____)\____)_| |_|
    (C)2013 Semtech

Description: Ping-Pong implementation

License: Revised BSD License, see LICENSE.TXT file include in the project

Maintainer: Miguel Luis and Gregory Cristian
*/
#include <string.h>
#include "board.h"
#include "radio.h"



#if defined( USE_BAND_433 )

#define RF_FREQUENCY                                434000000 // Hz

#elif defined( USE_BAND_470 )

#define RF_FREQUENCY                                470000000 // Hz

#elif defined( USE_BAND_868 )

#define RF_FREQUENCY                                868000000 // Hz

#elif defined( USE_BAND_915 )

#define RF_FREQUENCY                                915000000 // Hz

#else
    #error "Please define a frequency band in the compiler options."
#endif


#define TX_OUTPUT_POWER                             14        // dBm

#if defined( USE_MODEM_LORA )

#define LORA_BANDWIDTH                              7         //0: 7.8 khz
																															//1: 10.4 khz
																															//2: 15.6 khz
																															//3: 20.8 khz
																															//4: 31.2 khz
																															//5: 41.7 khz
																															//6: 62.5 khz
																															//7: 125 kHz,
                                                              //8: 250 kHz,
                                                              //9: 500 kHz,
                                                              //10: Reserved]
#define LORA_SPREADING_FACTOR                       12         // [SF7..SF12]
#define LORA_CODINGRATE                             2         // [1: 4/5,
                                                              //  2: 4/6,
                                                              //  3: 4/7,
                                                              //  4: 4/8]
#define LORA_PREAMBLE_LENGTH                        4         // Same for Tx and Rx  +4 is real length
#define LORA_SYMBOL_TIMEOUT                         500       // Symbols
#define LORA_FIX_LENGTH_PAYLOAD_ON                  false
#define LORA_IQ_INVERSION_ON                        false
#define RX_TIMEOUT_VALUE                            30000000
#define BUFFER_SIZE                                 9 			// Define the payload size here


#elif defined( USE_MODEM_FSK )

#define FSK_FDEV                                    25e3      // Hz
#define FSK_DATARATE                                50e3      // bps
#define FSK_BANDWIDTH                               50e3      // Hz
#define FSK_AFC_BANDWIDTH                           83.333e3  // Hz
#define FSK_PREAMBLE_LENGTH                         5         // Same for Tx and Rx
#define FSK_FIX_LENGTH_PAYLOAD_ON                   false

#else
    #error "Please define a modem in the compiler options."
#endif


uint16_t BufferSize = BUFFER_SIZE;
uint8_t Buffer[BUFFER_SIZE];



const uint8_t PingMsg[] = "PING";
const uint8_t PongMsg[] = "PONG";


States_t State = LOWPOWER;
States_t PreState = LOWPOWER;

int8_t RssiValue = 0;
int8_t SnrValue = 0;

/*!
 * Radio events function pointer
 */
static RadioEvents_t RadioEvents;

/*!
 * \brief Function to be executed on Radio Tx Done event
 */
void OnTxDone( void );

/*!
 * \brief Function to be executed on Radio Rx Done event
 */
void OnRxDone( uint8_t *payload, uint16_t size, int16_t rssi, int8_t snr );

/*!
 * \brief Function executed on Radio Tx Timeout event
 */
void OnTxTimeout( void );

/*!
 * \brief Function executed on Radio Rx Timeout event
 */
void OnRxTimeout( void );

/*!
 * \brief Function executed on Radio Rx Error event
 */
void OnRxError( void );

/**
 * Main application entry point.
 */
int main( void )
{
    
    // Target board initialisation
    BoardInitMcu( );
    BoardInitPeriph( );

	
    // Radio initialization
    RadioEvents.TxDone = OnTxDone;
    RadioEvents.RxDone = OnRxDone;
    RadioEvents.TxTimeout = OnTxTimeout;
    RadioEvents.RxTimeout = OnRxTimeout;
    RadioEvents.RxError = OnRxError;

    Radio.Init( &RadioEvents );

    Radio.SetChannel( RF_FREQUENCY );

		
	
#if defined( USE_MODEM_LORA )

    Radio.SetTxConfig( MODEM_LORA, TX_OUTPUT_POWER, 0, LORA_BANDWIDTH,
                                   LORA_SPREADING_FACTOR, LORA_CODINGRATE,
                                   LORA_PREAMBLE_LENGTH, LORA_FIX_LENGTH_PAYLOAD_ON,
                                   true, 0, 0, LORA_IQ_INVERSION_ON, 3000000);
    
    Radio.SetRxConfig( MODEM_LORA, LORA_BANDWIDTH, LORA_SPREADING_FACTOR,
                                   LORA_CODINGRATE, 0, LORA_PREAMBLE_LENGTH,
                                   LORA_SYMBOL_TIMEOUT, LORA_FIX_LENGTH_PAYLOAD_ON,
                                   0, true, 0, 0, LORA_IQ_INVERSION_ON, true );

#elif defined( USE_MODEM_FSK )

    Radio.SetTxConfig( MODEM_FSK, TX_OUTPUT_POWER, FSK_FDEV, 0,
                                  FSK_DATARATE, 0,
                                  FSK_PREAMBLE_LENGTH, FSK_FIX_LENGTH_PAYLOAD_ON,
                                  true, 0, 0, 0, 3000000 );
    
    Radio.SetRxConfig( MODEM_FSK, FSK_BANDWIDTH, FSK_DATARATE,
                                  0, FSK_AFC_BANDWIDTH, FSK_PREAMBLE_LENGTH,
                                  0, FSK_FIX_LENGTH_PAYLOAD_ON, 0, true,
                                  0, 0,false, true );

#else
    #error "Please define a frequency band in the compiler options."
#endif
    
 //   Radio.Rx( RX_TIMEOUT_VALUE );

State=LOWPOWER;
																	
																	
    while( 1 )
    {
        switch( State )
        {
        case RX:
		
						GpioWrite( &Led1_Rx, 0 ); 
						Radio.Rx( RX_TIMEOUT_VALUE );
						State = LOWPOWER;			
						//RtcDelayMs( 1000 );	
            break;
        case TX:
						
						GpioWrite( &Led2_Tx, 0);

						// Send the next PING frame   
						Buffer[0] = 0;
						Buffer[1] = 0;
						Buffer[2] = 0;
						Buffer[3] = 0;
						Buffer[4] = 0x01;
						Buffer[5] = 'P';
						Buffer[6] = 'E';
				    Buffer[7] = 'R';
				    Buffer[8] = 0xE8;
						

					
						// We fill the buffer with numbers for the payload 
		/*				for( i = 4; i < BufferSize; i++ )
						{
								Buffer[i] = i - 4;
						}*/
						DelayMs( 1 ); 
						Radio.Send( Buffer, BufferSize );
						//DelayMs( 100 ); 
						State = LOWPOWER;
            break;
        case RX_TIMEOUT:
        case RX_ERROR:
  
            State = LOWPOWER;
            break;
        case TX_TIMEOUT:
      
            State = LOWPOWER;
            break;
        case LOWPOWER:
        default:
				//	  Radio.Sleep( );
					//	GpioWrite( &Led1_Rx, 1); //LED off
					//	GpioWrite( &Led2_Tx, 1);
            // Set low power
            break;
        }
    
        TimerLowPowerHandler( ); 
    
    }
}

void OnTxDone( void )
{
	GpioWrite( &Led2_Tx, 1);
    Radio.Sleep( );
  //  State = LOWPOWER;
	State = TX;
	DelayMs( 200 ); 
}

void OnRxDone( uint8_t *payload, uint16_t size, int16_t rssi, int8_t snr )
{
    Radio.Sleep( );
//    BufferSize = size;
    memcpy( Buffer, payload, BufferSize );
			
    RssiValue = rssi;
    SnrValue = snr;
    State = RX;

	/*  if(Buffer[0]!='P')
		{	
			GpioWrite( &Led1_Rx, 1); //LED off
				
			return;
		} 
		*/
	  for (int i=0; i<3; i++)
		{
			GpioWrite( &Led1_Rx, 0); 
			GpioWrite( &Led2_Tx, 0);
			DelayMs(300);
			GpioWrite( &Led1_Rx, 1); //LED off
			GpioWrite( &Led2_Tx, 1);
			DelayMs(300);
		}
}

void OnTxTimeout( void )
{
		GpioWrite( &Led2_Tx, 1);
		Radio.Sleep( );
    //State = TX_TIMEOUT;
		State = TX;
		DelayMs( 200 ); 
}

void OnRxTimeout( void )
{
    Radio.Sleep( );
    //State = RX_TIMEOUT;
		State = RX;
//	  GpioWrite( &Led1_Rx, 1);
}

void OnRxError( void )
{
    Radio.Sleep( );
		State = RX;
//	  GpioWrite( &Led1_Rx, 1);
}


